from toolbox import Enum
from collections import namedtuple
import sys

from toolbox import setup_colorama
from toolbox.colorama import Fore, Back
setup_colorama()

"""
Methods
* load
* save
* run
"""

STEPS=41
ConversionMode = Enum('Moon', 'Sun','Always','Hidden')
GardenMode    = Enum('Moon', 'Sun')
GardenAttrs  = Enum('Nit','Pot','H20','Sun','Asc','Bio','Car','Fol','Nia','Pyr','Thi')
 
def str2enum(enum,str):
    idx = enum._keys.index(str)
    return enum[idx]

# *************************************************************************
#  
# *************************************************************************
fields = 'cid get_name give_name get_value give_value when_name when_value when_fun convo_type get_class give_class is_void when_opp'
class Conversion( namedtuple('Conversion', fields) ):
    """ This is an IMMUTABLE class """
    __slots__ = ()
    @classmethod
    def make(cls,default=None,def_bool=False,**kwargs):
        """ Construct the namedtuple from the specified keyword arguments,
        with every value considered optional, defaulting to None if it was
        not passed."""
        for key in cls._fields:
            if key.startswith('is_'):
                kwargs[key] = kwargs.get(key, default)
            else:
                kwargs[key] = kwargs.get(key, def_bool)
        return cls(**kwargs)  
    
    @property
    def name(self):
        """ Get the name of the converiosn like Nit_Pot. """
        if self.get_name:
            return "{0}_{1}".format(self.get_name,self.give_name)
        else:
            return "Conversion"
    
    @property
    def description(self,pointer="->"):
        """ Get the decription of the converison like Nit -> Asc """
        #TODO: Format aligned
        #TODO: Format for terminal
        descr = "{0}{1}{2}".format(self.get_name,pointer,self.give_name)
        return descr
    
    @property
    def description_full (self,pointer=" -> "):
        """ Get the decription of the converison like 10.0Nit -> 3.00 Asc """
        #TODO: Format aligned
        #TODO: Format for terminal
        if self.convo_type == ConversionMode.Hidden:
            return "{0: 2.2f} {1}{2}(nothing)".format(self.get_value, self.get_name, pointer)
        descr = "{0: 2.2f} {1}{2}{3: >5.2f} {4}".format(self.get_value, self.get_name, pointer, self.give_value, self.give_name)
        return descr
    
    @property
    def when(self):
        descr = "{0} {1} {2}".format(self.when_name,self.when_opp,self.when_value)
        return descr
    
    @property
    def class_name(self):
        return self.convo_type.key

# *************************************************************************
#  Extend list with usefull methods
# *************************************************************************
class ConversionList(list):
    """ This is a list of Conversions. """
    def __init__(self, name="Unnamed converion list", data=[]):
        list.__init__(self,data)
        self.name = name
        
    # http://code.activestate.com/recipes/576428-user-list-subclass/
    def __getslice__(self, i, j):
        return self.__getitem__(slice(i, j))
    def __setslice__(self, i, j, seq):
        return self.__setitem__(slice(i, j), seq)
    def __delslice__(self, i, j):
        return self.__delitem__(slice(i, j))

    def set_void(self,id,value=True):
        # TODO find id
        convo = filter(lambda item: item.cid==id, self)[0]
        if convo:
            idx = self.index(convo)
            self.__setitem__(idx, convo._replace(is_void=value) )
    
    def get_voids(self):
        items = filter(lambda item: item.is_void==True, self)
        voids = ConversionList("voids of "+self.name)
        for item in items:
            voids.append(item)
        return voids
        
    def get_by_id(self,id):
        """ get a conversion given his id """
        convo = filter(lambda item: item.cid==id, self)
        if len(convo):
            return convo[0]
        else:
            return None
                
    def print_console(self):
        """ Print on the console """
        print "{color}={name:^40}={reset}".format(name=self.name+" ("+str(len(self))+")",
            color=Back.GREEN,reset=Fore.RESET+Back.RESET)
        for convo in self:
            color = Fore.BLUE   if convo.convo_type == ConversionMode.Sun  else Fore.RESET
            color = Fore.GREEN   if convo.convo_type == ConversionMode.Moon else color
            color = Fore.YELLOW if convo.is_void else color
            if convo.convo_type == ConversionMode.Hidden:
                print "{color}*     {0}".format(convo.description_full,color=Fore.MAGENTA)
            else:
                active =" (void)" if convo.is_void else ""
                fmt="{color}id:{id:<2} {decr} {when} {klass:<6} {active} {pid}"
                print fmt.format(id=convo.cid, decr=convo.description_full, when=convo.when,
                                 color=color, active=active, klass=convo.class_name,pid=id(self))
            sys.stdout.write(Fore.RESET)


# *************************************************************************
#  Initialize available conversions from a file
# *************************************************************************
def load_conversions_from_file(fname=None):
    """ Create a list of conversion from another list or from a file """
    items = ConversionList("all")
    handle=file(fname, 'r')
    cid = 0
    for line in handle:
        fields = [x.strip() for x in line.split(' ') if len(x.strip()) > 0]
        if len(fields) and not line.startswith('#'):
            # get_name give_name get_value give_value when_name when_value when_type, convo_type
            # [0] 10.0 [1]Pot [2]2.50 [3]Bio [4]Sun [5]< [6]20
            get_value = float(fields[0])
            get_name  = fields[1]
            if fields[2] == 'Always':
                convo_type = ConversionMode.Hidden
                when_fun   = lambda x: True
                conversion = Conversion.make(convo_type=convo_type,is_void=False, 
                                              get_name=get_name,get_value=get_value,
                                              when_fun=when_fun)
            else:
                cid = cid +1
                give_value = float(fields[2])
                give_name  = fields[3]
                #TODO: Moon/sun/always
                #give_class = str2enum(GardenAttrs,give_name)
                when_name  = fields[4]
                when_opp   = fields[5]
                when_value = float(fields[6])
                if fields[5] == "<":
                    when_fun = lambda x : x < float(when_value) 
                else:
                    when_fun = lambda x : x > float(when_value)
                # ConverionMode = Enum('Moon', 'Sun','Always','Hidden')
                convo_type = ConversionMode.Always
                if when_name == "Sun":
                    convo_type = ConversionMode.Sun if int(when_value) >= 33 else ConversionMode.Moon
                
                conversion = conversion.make(convo_type=convo_type,is_void=False, 
                                             get_name=get_name, get_value=get_value,
                                             give_value=give_value, give_name=give_name,
                                             when_name=when_name, when_value=when_value,
                                             when_fun=when_fun,cid=cid,when_opp=when_opp)
            
            items.append(conversion)
    return items

import helpers
global all_conversions
all_conversions = load_conversions_from_file(helpers.get_data_file("thistle","convos.txt"))


# *************************************************************************
#  
# *************************************************************************
class ThistleGarden(object):
    def __init__(self,init_sun=33):
        self.mode = GardenMode.Moon if init_sun < 33 else GardenMode.Sun      
        print "TODO: Init"
   
    def print_on_console(self):
        print "Garden mode : {0} mode.".format(self.mode.key)
        pass 


# *************************************************************************
#  
# *************************************************************************
if __name__ == "__main__":
    all_conversions.set_void(1)
    all_conversions.set_void(7)
    all_conversions.set_void(8)
    all_conversions.set_void(15)
    all_conversions.set_void(20)
    all_conversions.print_console()
    
    voids =  all_conversions.get_voids()
    #TODO: This must be a filter
    #      maybe using generators??
    voids.set_void(1,False)
    voids.print_console()
    
    
