from toolbox import setup_colorama, Enum
from toolbox.colorama import Fore, Back
import types
setup_colorama()

ConversionMode = Enum('Moon', 'Sun','Always','Hidden')
GardenMode    = Enum('Moon', 'Sun')
GardenAttrs  = Enum('Nit','Pot','H20','Sun','Asc','Bio','Car','Fol','Nia','Pyr','Thi')


# *************************************************************************
#                      A class to represent a conversion
# *************************************************************************
class Conversion(object):
    """ Conversion(cid, get_name, give_name, ......., give_class, is_void, when_opp)"""
    __slot_fields__ = [(int,'cid',None,True),
                       (str,'get_name',None,True),
                       (int,'get_value',0,True),
                       (str,'give_name',"",True),
                       (float,'give_value',0.00,True),
                       (str,'when_name',"",True), 
                       (int,'when_value',0,True),
                       (str,'when_opp',"=",True),
                       (type(ConversionMode), 'convo_type', ConversionMode.Hidden, False),
                       (types.FunctionType,   'when_func',  lambda x: True, False),
                       (bool,'is_void', False, False),
                       (str,'notes', "", True),
                       (str,'name', None, False),]
    __slots__   = tuple([name    for (klass,name,default,readwrite) in __slot_fields__])
    __types__   = tuple([klass   for (klass,name,default,readwrite) in __slot_fields__])
    __defvals__ = tuple([default for (klass,name,default,readwrite) in __slot_fields__])
    del name, klass,default,readwrite
    
    def __init__(self,**argsKW):
        for k,v,klass in [(name,default,klass) for (klass,name,default,readwrite) in Conversion.__slot_fields__ if readwrite==True]:
            if argsKW.has_key(k):
                if type(argsKW[k]) == klass:
                    self.__setattr__(k,argsKW[k])
                else:
                    raise BaseException( "Invalid type '{0}' for filed '{1}'.".format(type(argsKW[k]),k) )
            else:
                self.__setattr__(k,v)
        self.__update_readonly_vars__()
    
    def __update_readonly_vars__(self):
        print "updatevars"
        self.name = "{0}_{1}".format(self.get_name,self.give_name) if self.get_name else None
        pass 

    def __setattr__(self, key, value):
        super(self.__class__, self).__setitem__(key, value)
        self.__update_readonly_vars__()
    
    @property
    def description(self,pointer="->"):
        """ Get the decription of the converison like Nit -> Asc """
        #TODO: Format aligned
        #TODO: Format for terminal
        descr = "{0}{1}{2}".format(self.get_name,pointer,self.give_name)
        return descr
    
    @property
    def description_full (self,pointer=" -> "):
        """ Get the decription of the converison like 10.0Nit -> 3.00 Asc """
        #TODO: Format aligned
        #TODO: Format for terminal
        if self.convo_type == ConversionMode.Hidden:
            return "{0: 2.2f} {1}{2}(nothing)".format(self.get_value, self.get_name, pointer)
        descr = "{0: 2.2f} {1}{2}{3: >5.2f} {4}".format(self.get_value, self.get_name, pointer, self.give_value, self.give_name)
        return descr
    
    def __len__(self):
        return len(Conversion.__slots__)

    def __getitem__(self, index):
        return getattr(self, self.__slots__[index])

    def __setitem__(self, index, value):
        """ convo[num_id]=8 """
        return setattr(self, self.__slots__[index], value)
    
    def todict(self):
        'Return a new dict which maps field names to their values'
        return {'cid': self.cid, 'get_name': self.get_name, 'give_name': self.give_name, 
                'get_value': self.get_value, 'give_value': self.give_value, 'when_name': self.when_name,
                'when_value': self.when_value, 'when_fun': self.when_fun, 'convo_type': self.convo_type,
                'get_class': self.get_class, 'give_class': self.give_class, 'is_void': self.is_void, 
                'when_opp': self.when_opp,'notes':self.notes}
    
    def __2repr__(self):
        #TODO
        return 'Conversion(cid=%r, get_name=%r, give_name=%r, get_value=%r, give_value=%r, when_name=%r, when_value=%r, when_fun=%r, convo_type=%r, get_class=%r, give_class=%r, is_void=%r, when_opp=%r)' % (self.cid, self.get_name, self.give_name, self.get_value, self.give_value, self.when_name, self.when_value, self.when_fun, self.convo_type, self.get_class, self.give_class, self.is_void, self.when_opp)

    def __eq__(self, other):
        return (isinstance(other, self.__class__) and self.cid==other.cid 
                and self.get_name==other.get_name and self.give_name==other.give_name
                and self.get_value==other.get_value and self.give_value==other.give_value
                and self.when_name==other.when_name and self.when_value==other.when_value
                and self.when_fun==other.when_fun and self.convo_type==other.convo_type
                and self.get_class==other.get_class and self.give_class==other.give_class
                and self.is_void==other.is_void and self.when_opp==other.when_opp 
                and self.notes==other.notes)
    def __ne__(self, other):
        return not self==other
    
    def __getstate__(self):
        return (self.cid, self.get_name, self.give_name, self.get_value, self.give_value, self.when_name,
                self.when_value, self.when_fun, self.convo_type, self.get_class, self.give_class, 
                self.is_void, self.when_opp, self.notes)
    def __setstate__(self, state):
        (self.cid, self.get_name, self.give_name, self.get_value, self.give_value, self.when_name, self.when_value, self.when_fun, 
         self.convo_type, self.get_class, self.give_class, self.is_void, self.when_opp,self.notes) = state


if __name__ == "__main__":
    import gtk
    w =gtk.Window()
    w.connect ('destroy', lambda *ignored: gtk.main_quit ())
    treeview = gtk.TreeView(model=None)
    w.add(treeview)
    
    w.show_all()
    
    
    